"""SNAPtoolbelt Configuration Commands"""
import logging

import progressbar

from arghandler import ArgumentHandler

from snap_profile.network import Network
from snap_profile.profile import Profile, SerialProfile, TcpProfile, MqttProfile

import snapstraction.probe

from snapstraction.node_cache import NodeCache

log = logging.getLogger(__name__)


def config(parser, context, args):
    """Configuration-related commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands(
        {
            'new': config_new,
            'copy': config_copy,
            'delete': config_delete,
            'list': config_list,
            'move': config_move,
            'scan': probe,
            'profile': profile_gitalike,
            'network': network_gitalike,
            'node_cache': node_cache
        })

    log.info("config commands")
    log.info("config context: %r", context)
    log.info("config args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def config_new(parser, context, args):
    """New Configuration-related commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands({'profile': new_profile, 'network': new_network})

    log.info("config new commands")
    log.info("config new context: %r", context)
    log.info("config new args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def config_delete(parser, context, args):
    """Delete Configuration-related commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands(
        {'profile': delete_profile,
         'network': delete_network})

    log.info("config delete commands")
    log.info("config delete context: %r", context)
    log.info("config delete args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def config_copy(parser, context, args):
    """Copy Configuration-related commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands({'profile': copy_profile, 'network': copy_network})

    log.info("config copy commands")
    log.info("config copy context: %r", context)
    log.info("config copy args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def config_move(parser, context, args):
    """Copy Configuration-related commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands({'profile': move_profile, 'network': move_network})

    log.info("config move commands")
    log.info("config move context: %r", context)
    log.info("config move args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def config_list(parser, context, args):
    """Listing Configuration-related commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands({'profile': list_profile, 'network': list_network})

    log.info("config list commands")
    log.info("config list context: %r", context)
    log.info("config list args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def new_profile(parser, context, args):
    """Create a new profile"""

    handler = ArgumentHandler()
    handler.add_argument("kind",
                         type=str.lower,
                         choices=("serial", "tcp", "mqtt"),
                         metavar="KIND",
                         help="Kind of profile to create (serial, tcp, mqtt)")
    handler.add_argument("to_name", metavar="NAME", help="Name of profile")
    handler.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="Overwrite the destination profile, if it exists")
    args = handler.parse_args(args)

    log.info("Creating a new %s profile named %s as a copy of default",
             args.kind, args.to_name)

    # UGLY!
    # TODO: replace with Profile __new__ factory?
    if args.kind == "serial":
        profile = SerialProfile()
    elif args.kind == "tcp":
        profile = TcpProfile()
    elif args.kind == "mqtt":
        profile = MqttProfile()

    if args.to_name in Profile.list():
        if args.force:
            log.warn("Destination profile %s already exists, overwriting",
                     args.to_name)
            Profile.delete(args.to_name)
        else:
            log.error("Destination profile %s already exists, aborting!",
                      args.to_name)
            exit(1)

    # Get the new name
    profile.name = args.to_name
    log.debug("New profile: %r", profile)
    profile.save()


def delete_profile(parser, context, args):
    """Delete a profile"""

    handler = ArgumentHandler()
    handler.add_argument('profile_name')

    args = handler.parse_args(args)

    log.info("Deleting profile %s", args.profile_name)

    Profile.delete(args.profile_name)


def copy_profile(parser, context, args):
    """Copy a profile"""

    handler = ArgumentHandler()
    handler.add_argument("from_profile")
    handler.add_argument("to_profile")
    handler.add_argument("-f", "--force", action="store_true")
    args = handler.parse_args(args)

    log.info("Copying profile %s to %s", args.from_profile, args.to_profile)

    profile = Profile.load(args.from_profile)

    if args.to_profile in Profile.list():
        if args.force:
            log.warn("Destination profile %s already exists, overwriting",
                     args.to_profile)
        else:
            log.error("Destination profile %s already exists, aborting!",
                      args.to_profile)
            exit(1)

    # Get the new name
    profile.name = args.to_profile
    log.debug("To profile: %r", profile)
    profile.save()


def move_profile(parser, context, args):
    """Move a profile"""

    handler = ArgumentHandler()
    handler.add_argument("from_profile")
    handler.add_argument("to_profile")
    handler.add_argument("-f", "--force", action="store_true")
    args = handler.parse_args(args)

    log.info("Renaming profile %s to %s", args.from_profile, args.to_profile)

    profile = Profile.load(args.from_profile)

    if args.to_profile in Profile.list():
        if args.force:
            log.warn("Destination profile %s already exists, overwriting",
                     args.to_profile)
        else:
            log.error("Destination profile %s already exists, aborting!",
                      args.to_profile)
            exit(1)

    # Get the new name
    profile.name = args.to_profile
    log.debug("To profile: %r", profile)
    profile.save()

    # Now remove the old one.
    Profile.delete(args.from_profile)


def list_profile(parser, context, args):
    """List all the available profiles"""

    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Listing profiles")

    map(context.output, Profile.list())


def profile_gitalike(parser, context, args):
    """git config-like interface for profiles"""

    handler = ArgumentHandler()

    handler.add_argument("profile_name")
    handler.add_argument("key", nargs='?', default=None)
    handler.add_argument("value", nargs='?', default=None)

    args = handler.parse_args(args)

    log.info("profile config")
    log.info("ARGS: %r", args)

    # Try to read the profile
    profile = Profile.load(args.profile_name)

    log.info("Profile: %r", profile)

    if args.profile_name != profile.name:
        log.warn("Profile %s does not exist, %s was loaded", args.profile_name,
                 profile.name)

    if args.key is None:
        log.debug("Displaying profile: %s", profile.name)
        context.output(vars(profile))
    elif args.value is None:
        log.debug("Displaying %s in profile: %s", args.key, profile.name)
        context.output({args.key: getattr(profile, args.key)})
    else:
        log.debug("Updating %s = %s in profile: %s", args.key, args.value,
                  profile.name)
        setattr(profile, args.key, args.value)
        context.output({args.key: getattr(profile, args.key)})
        profile.save()


def new_network(parser, context, args):
    """Create a network"""

    handler = ArgumentHandler()
    handler.add_argument("to_name")
    handler.add_argument("-f", "--force", action="store_true")
    args = handler.parse_args(args)

    log.info("Creating a new network named %s as a copy of default",
             args.to_name)

    # Clone the default
    net = Network.load('default')

    if args.to_name in Network.list():
        if args.force:
            log.warn("Destination network %s already exists, overwriting",
                     args.to_name)
        else:
            log.error("Destination network %s already exists, aborting!",
                      args.to_name)
            exit(1)

    net.name = args.to_name
    log.debug("New network: %r", net)
    net.save()


def list_network(parser, context, args):
    """List networks"""

    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Listing networks")

    map(context.output, Network.list())


def delete_network(parser, context, args):
    """Delete a network"""

    handler = ArgumentHandler()
    handler.add_argument('network_name')

    args = handler.parse_args(args)

    log.info("Deleting network %s", args.network_name)

    Network.delete(args.network_name)


def copy_network(parser, context, args):
    """Copy a network"""

    handler = ArgumentHandler()
    handler.add_argument('from_network')
    handler.add_argument('to_network')
    handler.add_argument("-f", "--force", action="store_true")
    args = handler.parse_args(args)

    log.info("Copying network from %s to %s", args.from_network,
             args.to_network)

    net = Network.load(args.from_network)

    if args.to_network in Network.list():
        if args.force:
            log.warn("Destination network %s already exists, overwriting",
                     args.to_network)
        else:
            log.error("Destination network %s already exists, aborting!",
                      args.to_network)
            exit(1)

    # Get the new name
    net.name = args.to_network
    log.debug("To network: %r", net)
    net.save()


def move_network(parser, context, args):
    """Move a network"""

    handler = ArgumentHandler()
    handler.add_argument('from_network')
    handler.add_argument('to_network')
    handler.add_argument("-f", "--force", action="store_true")
    args = handler.parse_args(args)

    log.info("Renaming network from %s to %s", args.from_network,
             args.to_network)

    net = Network.load(args.from_network)

    if args.to_network in Network.list():
        if args.force:
            log.warn("Destination network %s already exists, overwriting",
                     args.to_network)
        else:
            log.error("Destination network %s already exists, aborting!",
                      args.to_network)
            exit(1)

    # Get the new name
    net.name = args.to_network
    log.debug("To network: %r", net)
    net.save()

    Network.delete(args.from_network)


def network_gitalike(parser, context, args):
    """git config-like interface for networks"""

    handler = ArgumentHandler()

    handler.add_argument("network_name")
    handler.add_argument("key", nargs='?', default=None)
    handler.add_argument("value", nargs='?', default=None)

    args = handler.parse_args(args)

    log.info("network config")

    # Try to read the network
    network = Network.load(args.network_name)

    if args.network_name != network.name:
        log.warn("Network %s does not exist, %s was loaded", args.network_name,
                 network.name)

    if args.key is None:
        log.debug("Displaying network %s", network.name)
        context.output(vars(network))
    elif args.value is None:
        log.debug("Displaying %s in network %s", args.key, network.name)
        context.output({args.key: getattr(network, args.key)})
    else:
        log.debug("Updating %s = %s in network %s", args.key, args.value,
                  network.name)
        log.info("Updating network")
        setattr(network, args.key, args.value)
        context.output({args.key: getattr(network, args.key)})
        network.save()


def probe(parser, context, args):  # noqa
    """Probe for local SNAP connections, output on stdout as profile block"""

    handler = ArgumentHandler()

    handler.add_argument("-d",
                         "--dry-run",
                         action="store_true",
                         default=False,
                         help="Just scan, don't save new profiles")

    handler.add_argument("-f",
                         "--force",
                         action="store_true",
                         default=False,
                         help="Ignore read errors in result")

    args = handler.parse_args(args)

    log.info("Probing for local SNAP nodes")

    if context.timeout < 30:
        context.timeout = 180
        log.warning(
            "Bumping timeout to 180 seconds, scanning can take a little while...")

    widgets = [progressbar.FormatLabel('Scanning for nodes:'), ' ',
               progressbar.BouncingBar(marker=progressbar.RotatingMarker()),
               ' ', progressbar.Timer()]

    progress = progressbar.ProgressBar(widgets=widgets)
    progress.start()

    def on_progress(kind, count):
        widgets[0] = progressbar.FormatLabel('Scanning for %s nodes:' % kind)
        progress.update(count)

    def on_finished():
        progress.finish()

    new_profiles = snapstraction.probe.probe_serial_nodes(
        force=args.force,
        dry_run=args.dry_run,
        timeout=context.timeout,
        progress_cb=on_progress,
        finished_cb=on_finished)

    log.info("Got new profiles: %r", new_profiles)
    if new_profiles:
        map(context.output, new_profiles)


def node_cache(parser, context, args):
    """Node Cache-related commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands(
        {'clear': node_cache_clear,
         'show': node_cache_show})

    log.info("node cache commands")
    log.info("node cache context: %r", context)
    log.info("node cache args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def node_cache_clear(parser, context, args):
    """Remove one or all elements from the node cache"""

    handler = ArgumentHandler()
    handler.add_argument("address", help="address to remove or 'all'")
    args = handler.parse_args(args)

    if args.address.lower() == 'all':
        with NodeCache() as cache:
            cache.clear()

    else:
        with NodeCache() as cache:
            cache.discard(args.address)


def node_cache_show(parser, context, args):
    """Show all elements from the node cache"""

    with NodeCache() as cache:
        for addr in sorted(cache):
            context.output(addr)
