"""SNAPtoolbelt Serial Flash Commands"""
import logging
import progressbar

from arghandler import ArgumentHandler

import snapstraction.flash

log = logging.getLogger(__name__)


def flash(parser, context, args):
    """Serial Flasher commands"""
    handler = ArgumentHandler()

    # Add subcommands here.
    handler.set_subcommands({
        'firmware': firmware,
        'default_nv': default_nv,
        'erase_script': erase,
        'sniffer': sniffer,
    })

    log.info("flash commands")
    log.info("flash context: %r", context)
    log.info("flash args: %r", args)

    if context.profile.kind != 'serial':
        log.error(
            "Flash commands only work with direct serial access, %s not supported.",
            context.profile['snap_bridge_type'])
        exit(1)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def firmware(parser, context, args):  # noqa
    """Upload firmware via serial"""
    handler = ArgumentHandler()
    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")
    handler.add_argument('core', help="SNAP Core version (i.e. '2.6.9')")
    handler.add_argument('-f', '--file', default=None, help="Override firmware file path")

    args = handler.parse_args(args)

    log.info("Loading firmware for module %s, core %s", args.module, args.core)

    widgets = [progressbar.FormatLabel('Flash Firmware:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        progress.finish()

    progress.start()

    snapstraction.flash.firmware(context.profile, args.module, args.core, sniffer=False, override_file=args.file,
                                 progress_cb=on_progress, finished_cb=on_finished)


def sniffer(parser, context, args):  # noqa
    """Upload firmware via serial"""
    handler = ArgumentHandler()
    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")
    handler.add_argument('-f', '--file', default=None, help="Override firmware file path")

    args = handler.parse_args(args)

    log.info("Loading sniffer firmware for module %s", args.module)

    widgets = [progressbar.FormatLabel('Flash Sniffer:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        progress.finish()

    progress.start()

    snapstraction.flash.firmware(context.profile, args.module, '_', sniffer=True, override_file=args.file,
                                 progress_cb=on_progress, finished_cb=on_finished)


def default_nv(parser, context, args):
    """Reset NV Params via serial"""
    handler = ArgumentHandler()
    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")

    args = handler.parse_args(args)

    log.info("Defaulting NV Params for module %s", args.module)

    widgets = [progressbar.FormatLabel('Default NV Params:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        progress.finish()

    progress.start()

    snapstraction.flash.default_nv(context.profile,
                                   args.module,
                                   progress_cb=on_progress,
                                   finished_cb=on_finished)


def erase(parser, context, args):
    """Erase Script via serial"""
    handler = ArgumentHandler()
    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")

    args = handler.parse_args(args)

    log.info("Erasing SNAPpy Image for module %s", args.module)

    widgets = [progressbar.FormatLabel('Erase Script:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        progress.finish()

    progress.start()

    snapstraction.flash.erase_script(context.profile,
                                     args.module,
                                     progress_cb=on_progress,
                                     finished_cb=on_finished)
