"""SNAPtoolbelt Help Commands"""

import logging

from arghandler import ArgumentHandler

from snapconnect.snap import SnapConstantsHelper
from snapstraction import node, nvparam

log = logging.getLogger(__name__)


def help(parser, context, args):
    """Help-related commands"""
    handler = ArgumentHandler()
    handler.add_argument("subcommand", nargs='?', default=None)

    args, rest = handler.parse_known_args(args)

    help_subs = {
        'config': help_configuration,
        'quickstart': help_quickstart,
        'nvparam': help_nvparam,
        'stats': help_stats,
        'commands': help_commands
    }
    if args.subcommand in help_subs:
        help_subs[args.subcommand](parser, context, rest)
    else:
        help_help()


def help_help():
    print """
SNAPtoolbelt Help
=================

Topics (use `snap help TOPIC` to view):
  * quickstart
  * config
  * commands
  * nvparam
  * stats

"""


# Convert a help_string from NV_PARAM_HELP_STRINGS into a list of the help info for each bit
def extract_bits_help_from_help_string(help_string):
    return [help_line.split(' = ')[1]
            for help_line in help_string.split('\n')[1:]]


# Generate a dictionary of bit names to bit help for a given nv param
def bits_help_from_param_name(param_name):
    help_string = SnapConstantsHelper.NV_PARAM_HELP_STRINGS[
        nvparam.nvparam_lookup[param_name]['id']
    ]
    bits_help = extract_bits_help_from_help_string(help_string)
    # Make a bitfield help dictionary where the keys are the bit names as defined
    # in bitfields and the values are the bit help strings derived from the help string
    return dict(zip(nvparam.nvparam_lookup[param_name]['bitfields'].iterkeys(),
                    bits_help))

# Generate a list of the names of all nvparams that are bitmasks
bitmask_params = [
    param_name
    for (param_name, param_info) in nvparam.nvparam_lookup.iteritems()
    if 'bitfields' in param_info
]
# Dynamically generate help information about the various bitfield nvparams
bitfields_help = {
    param_name: bits_help_from_param_name(param_name)
    for param_name in bitmask_params
}


def help_nvparam(parser, context, args):
    """Print the nvparam help"""
    log.info("help nvparam context: %r", context)
    log.info("help nvparam args: %r", args)

    print "Available NV params:\n"

    for param_name, param_info in nvparam.nvparam_lookup.iteritems():
        param_id = param_info['id']
        try:
            param_string = SnapConstantsHelper.NV_PARAM_STRINGS[param_id]
            print '    %-44s %s (id %s)' % (param_name, param_string, param_id)
        except KeyError:
            # Ignore params without a help string
            break

        try:
            for bit_name, bit_id in param_info['bitfields'].iteritems():
                print '        %-40s %s (0x%03X)' % (
                    '.' + bit_name, bitfields_help[param_name][bit_name],
                    1 << bit_id)
        except KeyError:
            pass


def help_stats(parser, context, args):
    """Print the stats help"""
    log.info("help stats context: %r", context)
    log.info("help stats args: %r", args)

    print "Available stats:\n"

    print '    %-40s %s' % ('info', 'Query all info fields')
    print '    %-40s %s' % ('stat', 'Query all stat fields')
    print '    %-40s %s' % ('all', 'Query all info and stat fields')
    print ''

    for stat_name, stat_info in node.stat_lookup.iteritems():
        print '    %-40s %s %s' % (stat_name, stat_info['rpc'], stat_info['id']
                                   )


def help_commands(parser, context, args):
    """Help-related commands"""
    handler = ArgumentHandler()
    handler.add_argument("subcommand", nargs='?', default=None)

    args, rest = handler.parse_known_args(args)

    command_helps = {
        'config': help_commands_config,
        'rpc': help_commands_rpc,
        'node': help_commands_node,
        'mesh': help_commands_mesh,
        'serial': help_commands_serial
    }

    if args.subcommand in command_helps:
        command_helps[args.subcommand](parser, context, rest)
    else:
        help_commands_help()


def help_commands_help():
    print """
SNAPtoolbelt Command Help Topics
================================

Use `snap help commands TOPIC` to view.

* config
* rpc
* node
* mesh
* serial

"""


def help_commands_config(parser, context, args):
    """Print the config command help"""
    print """
### Configuration Commands

The configuration commands operate on Profiles and Networks.


#### Create a new profile

`snap config new profile {serial|tcp|mqtt} NAME [-f|--force]`

Create a new profile, named `NAME`.

You must specify what kind of profile to create: serial, tcp, or mqtt.

If the `-f` flag is specified, an existing profile/network with that name will be overwritten.

*Examples:*

Create a new serial profile named `sn132`
> `snap config new profile serial sn132`

Create a new mqtt profile named `e20`
> `snap config new profile mqtt e20`


#### Create a new network

`snap config new network NAME [-f|--force]`

Create a new network, named `NAME`.
If the `-f` flag is specified, an existing profile/network with that name will be overwritten.

*Examples:*

Overwrite the `lighting` network with a new one
> `snap config new network lighting -f`


#### List profiles/networks

`snap config list {profile|network}`

List the available profiles/networks.

*Examples:*

List Profiles
> `snap config list profile`


#### Delete a profile/network

`snap config delete {profile|network} NAME`

Remove the profile/network with name `NAME`

*Examples:*

Remove the `sn220` profile
> `snap config delete profile sn220`

Remove the `power` network
> `snap config delete network power`


#### Copy a profile/network

`snap config copy {profile|network} FROM TO [-f|--force]`

Copies the profile/network named `FROM` to `TO`. Use `-f` to overwrite an existing profile/network.

*Examples:*

Copy the `default` profile to `snapstick`
> `snap config copy profile default snapstick`

Copy the `lighting` network to `default`
> `snap config copy network lighting default -f`


#### Move a profile/network

`snap config move {profile|network} FROM TO [-f|--force]`

Moves the profile/network named `FROM` to `TO`. Use `-f` to overwrite an existing profile/network.

*Examples:*

Move the `sn220` profile to the `default`
> `snap config move profile sn220 default -f`

Move the `solar` network to `default`
> `snap config move network solar default -f`


#### Scan for serial profiles

`snap config scan [-d|--dry-run] [-f|--force] [-s|--snyf]`

Scans serial ports for SNAP nodes and creates profiles for any that don't already exist in your configuration.

Use `-d` to just scan without creating profiles.

Use `-f` to add nodes that have read errors.

Use `-s` to scan for sniffer nodes instead of regular nodes.

*Examples:*

Scan for all nodes
> `snap config scan -f`


#### Show a profile/network

`snap config {profile|network} NAME`

Display the profile/network `NAME`.

*Examples:*

Show the default profile
> `snap config profile default`

Show the default network
> `snap config network default`


#### Get a single value from a profile/network

`snap config {profile|network} NAME KEY`

Shows the `KEY` value from the profile/network `NAME`

*Examples:*

Show the `sn220` profile's device
> 'snap config profile sn220 device`

Show the `lighting` network`s encryption type
> `snap config network lighting encryption_type`


#### Set a single value in a profile/network

`snap config {profile|network} NAME KEY VALUE`

Sets `KEY` to `VALUE` in profile/network `NAME`

*Examples:*

Set the `sn220` profile's device to /dev/snap1
> `snap config profile sn220 device /dev/snap1`

Set the `solar` network`s channel to 7
> `snap config network solar channel 7`

#### Viewing the Node Cache

`snap config node_cache show`

Lists the entries currently in the Node Cache (nodes which will be available when autocompleting node addresses).

#### Removing entries from the Node Cache

`snap config node_cache clear {all|TARGET}`

Removes the specified entries from the Node Cache.

*Examples:*

Clear the entire cache
> `snap config node_cache clear all`

Remove 112233 from the cache
> `snap config node_cache clear 112233`


## Command Structure and Options

The SNAP Toolbelt commands are nested, similar to tools such as the Heroku Toolbelt.

Global options must come after the initial `snap`, but before the first subcommand:

`snap [-l|--log_level LOG_LEVEL] [-o|--output_format OUTPUT_FORMAT] [-p|--profile PROFILE] [-n|--network NETWORK] [-t|--timeout TIMEOUT] subcommand...`

(These options are not repeated in the [Tools](#tools) section below, but can be used with all commands.)

### Log Level

Use `-l [debug|info|warn|error|critical]` to specify the level at which the Toolbelt should log.

The SNAP Toolbelt always logs to STDERR. If you need to log to a file, you'll need to redirect STDERR using your shell of choice.

### Output Formats

The output is provided in Delimited Key-Value Pair ([DKVP][]) format.
For example, the output of a node info query in DKVP:

`address=001c2c1a7603fa1b,current_channel=4,current_netid=0x1c2c,nv_channel=4,nv_net_id=0x1c2c,nv_feature_bits=0x001f`

[mlr][MLR] can be used to convert DKVP into CSV or other formats.

JSON output is also possible (use `snap -o json ...`):

`{"address": "001c2c1a7603fa1b", "current_channel": 4, "current_netid": "0x1c2c", "nv_channel": 4, "nv_net_id": "0x1c2c", "nv_feature_bits": "0x001f"}`

### Profile

Use `-p PROFILE` to specify which profile you want to use. (`default` is the default.)

### Network

Use `-n NETWORK` to specify which network you want to use. (`default` is the default.)

### Timeout

Use `-t TIMEOUT` to specify the timeout for the command in seconds. (30 is the default, though some commands override this if it's too low. For instance, firmware uploads generally take longer than 30 seconds, so the timeout is bumped up for that command.)

"""


def help_commands_rpc(parser, context, args):
    """Print RPC command help"""
    print """
### Sending RPCs and Data

The SNAP Toolbelt provides access to the fundamental operation on a SNAP network: sending a message in the form of an RPC or Data Mode packet.
It also provides access to the callback forms for each "fire-and-forget" send.

*Notes:*
* Multicast Group, TTL, and DMCast Delay are all given default values in the Network configuration, but may be overridden per-command here.
* All target addresses must be in 6-character form: 0deCAF, 012345, &c.
* The special address "bridge" can be used to specify your connection's bridge node, even if the exact address is not known.
(One way to discover your bridge node's address is to use `snap node bridge info`.)

The Toolbelt will try to do the Right Thing when it comes to arguments:
* Arguments in quotes (`test`, `abc`) are parsed as strings.
* `""` and `''` are both parsed as empty strings.
* Integers are parsed as integers: (0x10 == 16, 100 == 100)
* Booleans (`True`, `False`) are parsed as booleans
* `None` is parsed as `None` (Pythonic None, not the string `'None'`)
* Strings beginning with x that look like hex data will be unhexlify'd: `xabcdef` == `'\xab\xcd\xef'`

#### Send Unicast RPC

`snap send rpc {u|unicast} TARGET FUNC ARG [ARG] [...]`

*Examples:*

Send `ping()` to the bridge node.
>`snap send rpc u bridge ping`

#### Send Multicast RPC

`snap send rpc {m|multicast} FUNC ARG [ARG] [...] [-g|--group GROUP] [-t|--ttl TTL]`

*Examples:*

Broadcast `hunt()`
>`snap send rpc m hunt`

#### Send Directed Multicast RPC

`snap send rpc {d|directed_multicast} TARGET[,TARGET][...] FUNC ARG [ARG] [...] [-g|--group GROUP] [-t|--ttl TTL] [-d|--delay DELAY]`

*Example:*

Send nodes `001122`, `334455`, and `667788` the RPC `getData(17, 'blue', 22)`
> `snap send rpc d 001122,334455,667788 getData 17 'blue' 22`

#### Send Unicast Data

`snap send data {u|unicast} TARGET PAYLOAD [--base64]`

If the `--base64` flag is specified, the PAYLOAD is assumed to be Base64-encoded and will be decoded before sending.

*Examples:*

Send `012345` the payload `This was a triumph.`
>`snap send data u 012345 "This was a triumph."`

#### Send Multicast Data

`snap send data {m|multicast} PAYLOAD [-g|--group GROUP] [-t|--ttl TTL] [--base64]`

If the `--base64` flag is specified, the PAYLOAD is assumed to be Base64-encoded and will be decoded before sending.

*Examples:*

Broadcast `I'm making a note here: HUGE SUCCESS` to group 7 with a ttl of 3.
>`snap send data m "I'm making a note here: HUGE SUCCESS" -g 7 -t 3`

#### Callback Unicast RPC

`snap call rpc {u|unicast} TARGET FUNC ARG [ARG] [...] [-c|--callback_name NAME]`

*Examples:*

Ask the bridge node `getInfo(5)`
>`snap call rpc u bridge getInfo 5`

Ask the bridge node `vmStat(6)`, expect it to return its result via `tellVmStat`
> `snap call rpc u bridge vmStat 6 --callback_name tellVmStat`

#### Callback Multicast RPC

`snap call rpc {m|multicast} TARGET[,TARGET][...] FUNC ARG [ARG] [...] [-g|--group GROUP] [--ttl TTL] [-c|--callback_name NAME]`

*Examples:*

Broadcast a request for `getInfo(5)`, expect responses from `123456` and `789abc`.
>`snap call rpc m 123456,789abc getInfo 5`

#### Callback Directed Multicast RPC

`snap call rpc {d|directed_multicast} TARGET[,TARGET][...] FUNC ARG [ARG] [...] [-g|--group GROUP] [-t|--ttl TTL] [-d|--delay DELAY] [-c|--callback_name NAME]`

*Examples:*

Directed Multicast a request for `getInfo(5)` to `123456` and `789abc`, ask them to delay their responses into 100ms windows.
>`snap call rpc d 123456,789abc getInfo 5 --delay 100`

#### Callback Unicast Data

*Coming Soon*

#### Callback Multicast Data

*Coming Soon*

"""


def help_commands_node(parser, context, args):
    """Print Node command help"""
    print """
### Node Tools

#### Query Node Information

`snap node TARGET info [-v]`

Get information about the target node. By default, queries:
* Address
* Current Channel (i.e. "Where is this node now?")
* Current Network ID
* NVParam Channel (i.e. "Where will this node go if it's rebooted?")
* NVParam Network ID
* Feature Bits

With the first `-v` flag, also queries:
* Device Name
* Device Type
* Script Name
* Script CRC
* Firmware Version

*Examples:*

Ask the bridge node for its basic information
> `snap node bridge info`

Ask node `123456` for more details
> `snap node 123456 info -v`

#### Node Traceroute

`snap node TARGET traceroute`

Performs a traceroute to the target node. Returns the round trip time and link quality information for each hop along the way.
See the SNAP Manual for more details on the traceroute process.

*Examples:*

Traceroute to `00aabb`
> `snap node 00aabb traceroute`


#### Build Script for Node

`snap node TARGET build SCRIPT [-I include [-I include]] [-p platform] [-o output_path]`

Build the script for the TARGET node. Uses SNAPbuild to compile the script to a SPY file for that node's module and core.

By default the output .spy/.map files are placed in the current working directory, use -o to override.

*Examples:*

Build SnapStick.py for the bridge node
> `snap node bridge build SnapStick.py`


#### Deploy Script

`snap node TARGET deploy SCRIPT [-I include [-I include]] [-p platform]`

Deploys the script to the TARGET node. Uses SNAPbuild to compile the script to a SPY file for that node's module and core.

If SCRIPT is already a SPY file, just upload it without rebuilding.

The output .spy/.map files are cached in ~/.snap/spycache.

*Examples:*

Deploy SnapStick.py to the bridge node
> `snap node bridge deploy SnapStick.py`


#### Query Script Information

`snap node TARGET script info`

Queries the script name and CRC.

*Examples:*

Ask the bridge for its script information
> `snap node bridge script info`


#### Erase Script

`snap node TARGET script erase`

Erases the script on the target node.

*Examples:*

Erase the script on node 987654
> `snap node 987654 script erase`


#### Load Script (OTA)

`snap node TARGET script load SPY_FILE`

Loads the SPY_FILE over-the-air on the TARGET node.

*Examples:*

Load SnapStick.spy on the bridge
> `snap node bridge script load SnapStick.spy`


#### Query Firmware Information

`snap node TARGET firmware info`

Query the firmware version on the TARGET node.

*Examples:*

Get firmware info for node `012345`
> `snap node 012345 firmware info`


#### Load Firmware (OTA)

`snap node TARGET firmware load SFI_FILE`

Performs an OTA firmware upgrade on the TARGET.

*Examples:*

Load `RF200_AES128_SnapV2.6.2.sfi` on node `012345`
> `snap node 012345 firmware load RF200_AES128_SnapV2.6.2.sfi`


#### Scan RF Energy

`snap node TARGET energy [-n|--num_queries QUERIES] [-d|--delay DELAY]`

Asks the node to check the RF energy on its channels.

If `-n QUERIES` is specified, do multiple queries and average the results.

If `-d DELAY` is specified, wait for DELAY seconds between queries.

*Examples:*

Ask the bridge node to scan 3 times, waiting 2 seconds between scans:
> `snap node bridge energy -n 3 -d 2`


#### Reboot Node

`snap node TARGET reboot [DELAY_MS]`

Ask the target node to reboot, with an optional delay of `DELAY_MS`.

*Examples:*

Reboot the bridge node
> `snap node bridge reboot`


#### NVParam Get

`snap node TARGET nvparam NVPARAM_ID`

Queries TARGET for the value of NVParam `NVPARAM_ID`. You can also reference NVParams by name, see `snap help nvparam` for details.

*Examples:*

Ask the bridge node for the value of NVParam 128
> `snap node bridge nvparam 128`

Ask node 123456 for its mesh routing max hop limit
> `snap node 123456 nvparam mesh.routing.max-hop-limit`

#### NVParam Set

`snap node TARGET nvparam NVPARAM_ID VALUE`

Set NVParam `NVPARAM_ID` to `VALUE` on the TARGET node. You can also reference NVParams by name, see `snap help nvparam` for details.

Values are parsed like RPC arguments.

*Examples:*

Set NVParam 110 to 'foo' on node `012345`
> `snap node 012345 nvparam set 110 'foo'`

Set the multicast forwarded groups on the bridge node to 5
> `snap node bridge nvparam set mcast.forwarded-groups 5`


#### NVParam Reset (OTA)

`snap node TARGET nvparam reset`

Does an OTA "Reset to factory default" of the NVParams.
*This command does not reboot the node. If you want the changes to take effect, you'll need to follow up with a reboot.*

*Example:*

Reset node 001122's NVParams to factory defaults
> `snap node 001122 nvparam reset`


#### Start Node Output Intercept

`snap node TARGET intercept start [--out-only|--err-only]`

Tell the TARGET node to start sending its STDOUT and/or STDERR to us.
Use the `snap mesh intercept` command to listen for the output after telling a node to start.

If `--out-only` is specified, send only STDOUT.

If `--err-only` is specified, send only STDERR.

*Examples:*

Ask node 001122 to start sending us both STDOUT and STDERR
> `snap node 001122 intercept start`


#### Stop Node Output Intercept

`snap node TARGET intercept stop`

Tell the TARGET node to stop sending its STDOUT and STDERR to us.

*Examples:*

Ask node 001122 to stop sending us output
> `snap node 001122 intercept stop`

#### Query Module Type

`snap node TARGET module_type`


#### Query VMStat

`snap node TARGET vmstat VMSTAT [ARG]`

Queries the specified VMSTAT on the TARGET node and breaks down the output.

*Examples:*

Ask the bridge for its vmStat(6)
> `snap node bridge vmstat 6`

Ask node 001122 for its vmStat(5, 2)
> `snap node 001122 vmstat 5 2`
"""


def help_commands_mesh(parser, context, args):
    """Print Mesh/Network Command Help"""
    print """
### (Mesh) Network Tools

Mesh tools deal with the mesh network as a whole, rather than individual nodes.


#### Ping Mesh

`snap mesh ping [-t|--timeout SECONDS]`

Send a multicast query to the network and await replies. The default timeout is 20 seconds.

*Examples:*

Ping the mesh, but only wait ~5 seconds for replies
> `snap mesh ping -t 5`


#### Directed Multicast Ping Mesh

`snap mesh dm_ping [-t|--timeout SECONDS]`

Send a multicast query to the network and await directed multicast replies. The default timeout is 20 seconds.

*Examples:*

Ping the mesh, but only wait ~5 seconds for replies
> `snap mesh dm_ping -t 5`


#### Listen for Intercepted Node Output

`snap mesh intercept [-t|--timeout TIMEOUT]`

After using the `snap node TARGET intercept start` command, use this
command to listen for their output.

By default, this command does not use the global timeout, it will run until you use CTRL-C to abort it.
If you specify the -t option, the intercept will run for that many seconds, then exit.

*Examples:*

Listen for intercepted output
> `snap mesh intercept`

Listen for intercepted output for 10 seconds, then quit
> `snap mesh intercept -t 10`

#### Listen for All Messages

`snap mesh listen [-t|--timeout TIMEOUT]`

If you don't have a real sniffer node available, you can always listen
for messages that happen to come your way (multicasts, for instance).

By default, this command does not use the global timeout, it will run until you use CTRL-C to abort it.
If you specify the -t option, it will listen for that many seconds, then exit.

*Examples:*

Listen for messages
> `snap mesh listen`

Listen for messages for one minute, then exit
> `snap mesh listen -t 60`


#### Find (and Move) Node

`snap mesh find TARGET [-m|--move NETWORK]`

Finds the target node by sweeping through channels.

*Examples:*

Move the bridge node to the lighting network:
> `snap mesh find bridge -m lighting`

Find node `123456`:
> `snap mesh find 123456`

Limitations:
* Cannot find nodes with different encryption settings from the Network/bridge.

"""


def help_commands_serial(parser, context, args):
    """Print Serial/Flash command help"""
    print """
### Serial-only Tools

Low-level serial flasher commands are also available. Use the `-p PROFILE` global option to select the target device.

On the E20 (and other gateways) you'll probably need to use `sudo` with these commands:

``sudo `which snap` flash erase_script SM220``

(The `` `which snap` `` portion provides sudo the full path to where you`ve installed the SNAP Toolbelt.)

#### Erase Script

`snap flash erase_script MODULE`

Erase any script on the target module.

*Examples:*
>`snap flash erase_script SS200`

#### Factory Default NV Params

`snap flash default_nv MODULE`

Restore the NV Params to the factory defaults.

*Examples:*
>`snap flash default_nv SS200`

#### Upload Firmware

`snap flash firmware load MODULE VERSION [--snyf] [-f|--file SFI_FILE]`

Load the specified firmware on the module through the bootloader. (Not over-the-air.)
If the `snap-sdk` package is installed, attempt to use it to look up the firmware file by `MODULE` and `VERSION`.
If the `--snyf` option is specified, look up the sniffer firmware instead.
If it's not installed, an SFI file path can be provided via the `-f` option.

*Examples:*
>`snap flash firmware load SM220UF1 2.6.2`

>`snap flash firmware load SS200 1.3.0 --snyf`

>`snap flash firmware load SS200 2.6.2 -f ./sfifiles/RF200_AES128_SnapV2.6.2.sfi`
"""


def help_configuration(parser, context, args):
    """Print configuration help"""
    print """
## Configuration

The SNAP Toolbelt stores its configuration in a '.snap' folder under your user folder:
- %USERPROFILE% on Windows, e.g. "C:\\Users\\YourName\\.snap"
- ~/.snap on Linux/OS X

The SNAP Toolbelt needs to know a few things in order to talk to your SNAP networks:
It needs to know what kind of connection to use (the Profile),
and it needs to know some information about how your network is set up (the Network).

The configuration (and some other data) is stored in a few files:
* profiles.cfg - SNAP Profiles, or "how should I talk to your SNAP networks?"
* networks.cfg - SNAP Networks, or "What networks do you talk to, and how are they configured?"

### Profiles

SNAP Profiles are the set of parameters needed to connect to a SNAP Network.

There are three kinds of Profiles: Serial, TCP, and MQTT.

#### Serial Profiles

A Serial Profile is used when you're talking to a serial node such as a SS200, SN220, or SN-171 Protoboard.

Serial Profiles include:
* device - Device string, can be a COM identifier on Windows (such as `COM1`), `USB0` for SN-132 paddleboards in USB mode, `SnapStick0` for SS200s in USB mode, or a device path like `/dev/snap1` on POSIX systems.
* address - Licensed address to use, can be left blank to use the default.

#### TCP Profiles

A TCP Profile is used when you're talking to a SNAP Connect instance that is accepting TCP connections.

TCP Profiles include:
* host - Hostname or IP address
* port - Port to connect to (defaults to 48625)
* ssl - Should this connection use SSL? (True or False)
* username - User to authenticate as
* password - Password to authenticate with
* address - Licensed address to use, can be left blank to use the default.

#### MQTT Profiles

An MQTT Profile is used when you're talking to a SNAP2MQTT instance via an MQTT gateway.

MQTT Profiles include:
* host - Hostname or IP address
* port - Port to connect to (defaults to 48625)
* ca_certs - path to client SSL certificates
* address - SNAP2MQTT's address (default is '000080')

### Networks

SNAP Networks are the parameters that all nodes in a network need to agree on in order to communicate:
* Channel: Integer from 0-15, defaults to 4
* Network ID: Defaults to 0x1c2c
* Encryption Type (encryption_type): None, Basic, or AES-128, defaults to None
* Encryption Key (encryption_key): String, up to 16 characters, defaults to ''
* Feature Bits: Defaults to 0x0100
"""


def help_quickstart(parser, context, args):
    """Print the quickstart guide"""
    print """
### QuickStart

If you're using a serial node such as an SS200 or SN220, the Toolbelt can scan for your node(s):

```
$ snap config scan
```

If you get a message about a `Read error`, you can bypass it by adding the `-f`/`--force` flag.
(Read errors may just mean "This node's actually a SNAP Sniffer, we were looking for non-Sniffer nodes.", or "It was busy when we tried to talk to it.")

The Toolbelt will automagically add new nodes to your configuration.

To make one of them your default, move it (with `-f` to overwrite the existing default):
```
$ snap config move profile SNAPstick0 default -f
```

### Autocompletion

A bash-completion script is provided with the toolbelt. To access it, use `snap autocomplete`:

```sh
$ source $(snap autocomplete)
```

The script will attempt to tab-complete commands, profiles, networks, and nodes you've talked to recently.

"""
