# -*- coding: utf-8 -*-
"""SNAPtoolbelt Low Level Commands"""

import logging

from arghandler import ArgumentHandler

import snapstraction.gus
from snapstraction.util import parse_value_for_snap, validate_snap_address
import snapstraction.low_level

log = logging.getLogger(__name__)


def call(parser, context, args):
    """Functions for callback rpc/mcast/dmcast"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'rpc': callback_rpc,
        # TODO: Re-enable this once it's actually implemented
        # 'data': callback_data,
    })

    log.info("callback commands")
    log.info("callback context: %r", context)
    log.info("callback args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def callback_rpc(parser, context, args):
    """Issue a callback_rpc to the node at the given address"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'u': callback_ucast_rpc,
        'unicast': callback_ucast_rpc,
        'm': callback_mcast_rpc,
        'multicast': callback_mcast_rpc,
        'd': callback_dmcast_rpc,
        'dm': callback_dmcast_rpc,
        'directed_multicast': callback_dmcast_rpc
    })

    log.info("callback RPC commands")
    log.info("callback RPC context: %r", context)
    log.info("callback RPC args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def callback_ucast_rpc(parser, context, args):
    """Issue a callback unicast RPC"""

    handler = ArgumentHandler()
    handler.add_argument(
        'target_address',
        help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('func', help="function name")
    handler.add_argument('-c', '--callback_name', default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    log.info(
        "Callback_rpc function %s, target %s expecting %r with args: %r",
        args.func, args.target_address, args.callback_name, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.callback_ucast_rpc,
                              args.target_address, args.func, *rest,
                              callback_name=args.callback_name)
        context.output(result)


def callback_mcast_rpc(parser, context, args):
    """Issue a callback_mcast to the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument('targets', help="Target Addrs")
    handler.add_argument('func', help="function name")
    handler.add_argument('-g',
                         '--group',
                         type=int,
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-c', '--callback_name', default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    args.targets = [addr.strip() for addr in args.targets.split(',')]

    log.info(
        "Callback_mcast targets %s, group %r, ttl %r, function %s, expecting %r with args: %r",
        args.targets, args.group, args.ttl, args.func, args.callback_name,
        rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.callback_mcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group, ttl=args.ttl,
                              callback_name=args.callback_name)
        context.output(result)


def callback_dmcast_rpc(parser, context, args):
    """Issue a callback_dmcast to the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument('targets', help="Target Addrs")
    handler.add_argument('func', help="function name")
    handler.add_argument('-g',
                         '--group',
                         type=int,
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-d',
                         '--delay_factor',
                         type=int,
                         help="delay factor",
                         default=None)
    handler.add_argument('-c', '--callback_name', default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    args.targets = [addr.strip() for addr in args.targets.split(',')]

    log.info(
        "Callback_dmcast targets %s, group %r, ttl %r, delay %r, function %s, expecting %r with args: %r",
        args.targets, args.group, args.ttl, args.delay_factor, args.func,
        args.callback_name, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.callback_dmcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group,
                              ttl=args.ttl,
                              delay_factor=args.delay_factor,
                              callback_name=args.callback_name)
        context.output(result)


# This is awaiting implementation in the snapstraction layer
# Note that there is no path from the call command to reach this yet.
# TODO: Remove pragma when snapstraction implements this...
def callback_data(parser, context, args):  # pragma: no cover
    """Issue a data mode packet and expect a response"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'u': callback_ucast_data,
        'unicast': callback_ucast_data,
        'm': callback_mcast_data,
        'multicast': callback_mcast_data,
    })

    log.info("callback RPC commands")
    log.info("callback RPC context: %r", context)
    log.info("callback RPC args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


# This is awaiting implementation in the snapstraction layer
# Note that there is no path from the call command to reach this yet.
# TODO: Remove pragma when snapstraction implements this...
def callback_ucast_data(parser, context, args):  # pragma: no cover
    """Issue a callback unicast data"""

    handler = ArgumentHandler()
    handler.add_argument(
        'target_address',
        help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('data', help="data to send")
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")
    handler.add_argument('-c', '--callback_regex', default=r'^ACK$')

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    log.info(
        "Callback_ucast_data: target %s, payload %r, expecting %r",
        args.target_address, args.data, args.callback_regex)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.callback_ucast_data,
                              args.target_address,
                              args.data,
                              callback_regex=args.callback_regex,
                              base64=args.base64)
        context.output(result)


# This is awaiting implementation in the snapstraction layer
# Note that there is no path from the call command to reach this yet.
# TODO: Remove pragma when snapstraction implements this...
def callback_mcast_data(parser, context, args):  # pragma: no cover
    """Issue a callback_mcast data packet"""
    handler = ArgumentHandler()
    handler.add_argument('targets', help="Target Addrs")
    handler.add_argument('data', help="data to send")
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")
    handler.add_argument('-c', '--callback_regex', default=r'^ACK$')
    handler.add_argument('--group',
                         type=int,
                         help="multicast group",
                         default=None)
    handler.add_argument('--ttl', type=int, help="multicast ttl", default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    args.targets = [addr.strip() for addr in args.targets.split(',')]

    log.info(
        "Callback_mcast_data targets %s, payload %r, expecting %r",
        args.targets, args.data, args.callback_regex)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.callback_mcast_data,
                              args.targets,
                              args.data,
                              group=args.group,
                              ttl=args.ttl,
                              callback_regex=args.callback_regex,
                              base64=args.base64)
        context.output(result)


def send(parser, context, args):
    """Functions for callback rpc/mcast/dmcast"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({'rpc': send_rpc, 'data': send_data, })

    log.info("send commands")
    log.info("send context: %r", context)
    log.info("send args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def send_rpc(parser, context, args):
    """Issue a callback_rpc to the node at the given address"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'u': ucast_rpc,
        'unicast': ucast_rpc,
        'm': mcast_rpc,
        'multicast': mcast_rpc,
        'd': dmcast_rpc,
        'dm': dmcast_rpc,
        'directed_multicast': dmcast_rpc
    })

    log.info("callback RPC commands")
    log.info("callback RPC context: %r", context)
    log.info("callback RPC args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def send_data(parser, context, args):
    """Issue a data mode packet the node at the given address"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'u': ucast_data,
        'unicast': ucast_data,
        'm': mcast_data,
        'multicast': mcast_data,
    })

    log.info("send data commands")
    log.info("send data context: %r", context)
    log.info("send data args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def ucast_rpc(parser, context, args):
    """Issue a fire-and-forget RPC to the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument(
        'target_address',
        help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('func', help="function name")

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    log.info("RPC function %s, target %s, args %r", args.func,
             args.target_address, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.ucast_rpc,
                              args.target_address, args.func, *rest)
        context.output(result)


def mcast_rpc(parser, context, args):
    """Issue a fire-and-forget MCAST RPC to the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument('--group',
                         type=int,
                         help="multicast group",
                         default=None)
    handler.add_argument('--ttl', type=int, help="multicast ttl", default=None)
    handler.add_argument('func', help="function name")

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    log.info("MCAST RPC group %r, ttl %r, func %s, args %r", args.group,
             args.ttl, args.func, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.mcast_rpc,
                              args.func, *rest, group=args.group, ttl=args.ttl)
        context.output(result)


def dmcast_rpc(parser, context, args):
    """Issue a fire-and-forget DMCAST RPC to the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument('targets', help="Target Addrs")
    handler.add_argument('--group',
                         type=int,
                         help="multicast group",
                         default=None)
    handler.add_argument('--ttl', type=int, help="multicast ttl", default=None)
    handler.add_argument('--delay_factor',
                         type=int,
                         help="delay factor",
                         default=None)
    handler.add_argument('func', help="function name")

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    # Split targets
    args.targets = [s.strip() for s in args.targets.split(',')]

    log.info(
        "DMCAST RPC targets %r, group %r, ttl %r, delay %r, func %s, args %r",
        args.targets, args.group, args.ttl, args.delay_factor, args.func, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.dmcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group, ttl=args.ttl, delay_factor=args.delay_factor)
        context.output(result)


def ucast_data(parser, context, args):
    """Issue a fire-and-forget Data packet to the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument(
        'target_address',
        help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('data', help="data to send")
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")

    args = handler.parse_args(args)

    address = validate_snap_address(args.target_address)

    log.info("Unicast data -> target %s, payload %r", address, args.data)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.ucast_data,
                              address,
                              args.data,
                              base64=args.base64)
        context.output(result)


def mcast_data(parser, context, args):
    """Issue a fire-and-forget MCAST Data packet to the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument('--group',
                         type=int,
                         help="multicast group",
                         default=None)
    handler.add_argument('--ttl', type=int, help="multicast ttl", default=None)
    handler.add_argument('data', help="data to send")
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")

    args = handler.parse_args(args)

    log.info("Multicast data -> group %r, ttl %r, payload %r", args.group,
             args.ttl, args.data)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.mcast_data,
                              args.data,
                              base64=args.base64,
                              group=args.group,
                              ttl=args.ttl)
        context.output(result)
