"""SNAPtoolbelt Mesh/Network Commands"""
from __future__ import print_function

import logging

import os

import progressbar

from arghandler import ArgumentHandler

from snap_profile.network import Network

import snapstraction.mesh
import snapstraction.node_finder

log = logging.getLogger(__name__)


def mesh(parser, context, args):
    """Mesh-related subcommands"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'ping': ping,
        'dm_ping': dm_ping,
        'find': find,
        'intercept': intercept_listen,
        'listen': listen,
        'topology': topology,
    })

    log.info("mesh commands")
    log.info("mesh context: %r", context)
    log.info("mesh args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def ping(parser, context, args):
    """Mcast ping the mesh"""
    handler = ArgumentHandler()

    handler.add_argument("-t",
                         "--timeout",
                         default=20,
                         type=int,
                         help="Time to wait for responses")

    args = handler.parse_args(args)

    log.info("Ping the mesh")

    widgets = [progressbar.FormatLabel('Unicast Ping'), ' ',
               progressbar.FormatLabel('Found 0 ()'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=args.timeout)
    progress.start()

    def on_progress(seconds, found_count, last_addr):
        widgets[2] = progressbar.FormatLabel('Found %d (%s)' %
                                             (found_count, last_addr))
        progress.update(seconds)

    def on_finished():
        progress.finish()

    with context.gus() as gus:
        result = gus.run_sync(args.timeout + 10,
                              snapstraction.mesh.unicast_ping,
                              args.timeout,
                              context.output,
                              progress_cb=on_progress,
                              finished_cb=on_finished)
        context.output(result)


def dm_ping(parser, context, args):
    """Ping that uses dmcast for the response instead of unicast"""
    handler = ArgumentHandler()

    handler.add_argument("-t",
                         "--timeout",
                         default=20,
                         type=int,
                         help="Time to wait for responses")
    args = handler.parse_args(args)

    log.info("Ping the mesh")

    widgets = [progressbar.FormatLabel('Directed Multicast Ping'), ' ',
               progressbar.FormatLabel('Found 0 ()'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=args.timeout)
    progress.start()

    def on_progress(seconds, found_count, last_addr):
        widgets[2] = progressbar.FormatLabel('Found %d (%s)' %
                                             (found_count, last_addr))
        progress.update(seconds)

    def on_finished():
        progress.finish()

    with context.gus() as gus:
        result = gus.run_sync(args.timeout + 10,
                              snapstraction.mesh.dmcast_ping,
                              args.timeout,
                              context.output,
                              progress_cb=on_progress,
                              finished_cb=on_finished)
        context.output(result)


def intercept_listen(parser, context, args):
    """Listen for intercepted node output"""
    handler = ArgumentHandler()
    handler.add_argument('-t',
                         '--timeout',
                         type=int,
                         default=None,
                         help='How many seconds to listen (default: forever)')
    args = handler.parse_args(args)

    log.info("Trying to listen for intercepted output")

    with context.gus() as gus:
        gus.run_sync(None,
                     snapstraction.mesh.listen_for_intercepted_output,
                     context.output,
                     _timeout=args.timeout)


def find(parser, context, args):
    """find the target node"""
    handler = ArgumentHandler()
    handler.add_argument('targets',
                         nargs='?',
                         default=None,
                         help="Node(s) to find")
    handler.add_argument('-l',
                         '--list',
                         dest="list_file",
                         help="File containing list of nodes to find")
    handler.add_argument('-m', '--move', help="Profile to move nodes to.")
    args = handler.parse_args(args)

    if args.list_file:
        # do expansion of list file name/path
        list_file_path = os.path.join(os.path.curdir, os.path.expanduser(args.list_file))
        targets = snapstraction.node_finder.get_targets_from_file(list_file_path)
    elif args.targets:
        targets = args.targets.split(',')
    else:
        log.error("Must provide either comma-separated target addresses (123456,234567,...) or a file with a list of targets, one per line")
        raise Exception("No targets provided")

    # Need to allow a lot more time for finding nodes
    if context.timeout < 60:
        log.warn(
            "Bumping timeout to 300 seconds, finding nodes can take a while...")
        context.timeout = 300

    # load the network data to pass down
    move_to_network = Network.load(args.move) if args.move else None

    log.info("finding node(s), optionally moving to target network...")

    with context.gus() as gus:
        result = gus.run_sync_with_data(context.timeout, snapstraction.node_finder.finder,
                                        targets, move_to_network=move_to_network)
        context.output(result)


def topology(parser, context, args):
    """Take the topology of the mesh"""
    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Get the topology of the mesh")

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.mesh.poll_topology)
        context.output(result)


def listen(parser, context, args):
    """Listen to packets received"""
    handler = ArgumentHandler()
    handler.add_argument('-t',
                         '--timeout',
                         type=int,
                         default=None,
                         help='How many seconds to listen (default: forever)')
    args = handler.parse_args(args)

    log.info("Listening to the mesh")

    with context.gus() as gus:
        gus.run_sync(None,
                     snapstraction.mesh.listen,
                     context.output,
                     _timeout=args.timeout)
