# -*- coding: utf-8 -*-
"""SNAPtoolbelt Node commands"""

import logging

from collections import OrderedDict

import os
import sys

import progressbar

from arghandler import ArgumentHandler

from snap_profile.network import Network

import snapstraction.gus
import snapstraction.build
import snapstraction.node
import snapstraction.nvparam
import snapstraction.node_finder
from snapstraction.node_cache import NodeCache

from snapstraction.util import parse_value_for_snap, validate_snap_address, clean_up
from .util import mkdir_p

log = logging.getLogger(__name__)


class IncompleteResultError(Exception):
    pass


def update_cache(context):
    """Add the context's target address to the node cache"""
    with NodeCache() as cache:
        cache.add(context.target_address)


def node(parser, context, args):
    """Node-related subcommands"""
    handler = ArgumentHandler()

    # All node commands take a single target address
    handler.add_argument(
        "target_address",
        help="6-char node address, or 'bridge' for the bridge node")

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({
        'build': build,
        'deploy': deploy,
        'energy': energy,
        'firmware': firmware,
        'info': info,
        'intercept': intercept,
        'module_type': module_type,
        'move': move,
        'nvparam': nvparam,
        'ping': ping,
        'reboot': reboot,
        'script': script,
        'stats': stats,
        'traceroute': traceroute,
        'vmstat': vmstat
    })

    log.info("node commands")
    log.info("node context: %r", context)
    log.info("node args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings

    def ctx(node_context):
        """Update the context for node commands"""
        # Check the address
        address = validate_snap_address(node_context.target_address)

        if address is None:
            raise Exception("%s is not a valid 6 hex char address" %
                            node_context.target_address)

        # Update the context
        node_context.target_address = address

        # Merge the parent context with the node context, using vars to get dicts
        vars(context).update(vars(node_context))
        return context

    handler.run(args, context_fxn=ctx)


def build(parser, context, args):  # noqa
    """Build the SCRIPT for the NODE"""
    handler = ArgumentHandler()
    handler.add_argument('script', help="SNAPpy script to deploy")
    handler.add_argument(
        '-p',
        '--platform',
        help="Node Platform: value of 'platform' constant at build time, defaults to module type",
    )
    handler.add_argument('-I',
                         '--Include',
                         dest='include',
                         default=[],
                         metavar='IMPORT_PATH',
                         help='Allow imports from additional path',
                         action='append')
    handler.add_argument(
        "-o",
        "--output",
        dest="output",
        metavar="SPY",
        help="Path of SNAPpy script image (and map file), defaults to ./SCRIPT.spy and ./SCRIPT.map. If this doesn't already end in '.spy', '.spy' will be appended.",
    )
    args = handler.parse_args(args)

    data = OrderedDict()

    args.script = os.path.abspath(os.path.expanduser(args.script))

    # Need to build it.
    # First, find out what module/core to use
    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.get_node_core_module,
                              context.target_address)

        core = result.get('core', None)
        module = result.get('module', None)

        log.info("Node %s is a(n) %s running Core %s", context.target_address,
                 module, core)

        data['core'] = core
        data['module'] = module

    if not core or not module:
        log.error("Could not determine target's core and module")
        exit(1)

    data['includes'] = args.include

    if args.output:
        output_path = args.output
    else:
        output_path = os.path.join(
            '.', os.path.splitext(os.path.basename(args.script))[0])

    data['output_dir'] = os.path.dirname(output_path)

    crc = snapstraction.build.call(args.script,
                                   module,
                                   core,
                                   includes=args.include,
                                   platform=args.platform,
                                   output_path=output_path)

    data['crc'] = crc

    spy_out = os.path.abspath(os.path.splitext(output_path)[0] + '.spy')
    data['spy'] = spy_out
    data['map'] = os.path.abspath(os.path.splitext(output_path)[0] + '.map')

    update_cache(context)
    context.output(data)


def deploy(parser, context, args):  # noqa
    """Deploy the SCRIPT to the NODE"""
    handler = ArgumentHandler()
    handler.add_argument('script', help="SNAPpy script to deploy")
    handler.add_argument(
        '-p',
        '--platform',
        help="Node Platform: value of 'platform' constant at build time, defaults to module type",
    )
    handler.add_argument('-I',
                         '--Include',
                         dest='include',
                         default=[],
                         metavar='IMPORT_PATH',
                         help='Allow imports from additional path',
                         action='append')
    args = handler.parse_args(args)

    data = OrderedDict()

    args.script = os.path.abspath(os.path.expanduser(args.script))

    # If the script is a SPY file, just load it.
    if os.path.splitext(args.script)[1].lower() == '.spy':
        data['spy'] = args.script
        result = ota_script_upload(context, args.script)
        data.update(**result)
        update_cache(context)
        context.output(data)

    elif os.path.splitext(args.script)[1].lower() == '.py':
        # Need to build it.
        # First, find out what module/core to use
        with context.gus() as gus:
            result = gus.run_sync(context.timeout,
                                  snapstraction.node.get_node_core_module,
                                  context.target_address)

            core = result.get('core', None)
            module = result.get('module', None)

            log.info("Node %s is a(n) %s running Core %s",
                     context.target_address, module, core)

            data['core'] = core
            data['module'] = module

        if not core or not module:
            log.error("Could not determine target's core and module")
            exit(1)

        data['includes'] = args.include

        spycache = os.path.expanduser(os.path.join('~', '.snap', 'spycache'))
        data['output_dir'] = spycache
        mkdir_p(spycache)

        output_path = os.path.join(
            spycache, os.path.splitext(os.path.basename(args.script))[0])

        crc = snapstraction.build.call(args.script,
                                       module,
                                       core,
                                       includes=args.include,
                                       platform=args.platform,
                                       output_path=output_path)

        data['crc'] = crc

        spy_out = os.path.abspath(os.path.splitext(output_path)[0] + '.spy')
        data['spy'] = spy_out
        data['map'] = os.path.abspath(os.path.splitext(output_path)[0] +
                                      '.map')
        result = ota_script_upload(context, spy_out)
        data.update(**result)
        update_cache(context)
        context.output(data)


def info(parser, context, args):
    """Get info from the node at the given address"""
    handler = ArgumentHandler()
    handler.add_argument(
        '-v',
        '--verbosity',
        action='append_const',
        const='v',
        default=[],
        help="verbosity of information (repeat for more info)")
    args = handler.parse_args(args)

    # Parse the verbosity to a numeric based on how many consummate vs
    args.verbosity = len(args.verbosity)

    log.debug("CONTEXT: %r", context)
    log.debug("ARGS: %r", args)

    log.info("Getting info from %s with verbosity level %s!",
             context.target_address, args.verbosity)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.get_info,
                              context.target_address,
                              verbosity=args.verbosity)
        errors = result.get('_errors', 0)
        del result['_errors']
        update_cache(context)
        context.output(result)
        if errors:
            raise IncompleteResultError()


def traceroute(parser, context, args):
    """Perform a traceroute on the target node"""
    handler = ArgumentHandler()

    args = handler.parse_args(args)

    log.info("Trying to perform traceroute on node %s", context.target_address)
    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.node.traceroute,
                              context.target_address)
        update_cache(context)
        context.output(result)


def nvparam(parser, context, args):
    """NVParam Git-alike subcommand"""
    handler = ArgumentHandler()

    handler.add_argument("nvparam")
    handler.add_argument("value", nargs='?', default=None)

    args = handler.parse_args(args)

    if args.nvparam.lower() == 'reset':
        log.info("Resetting NVParams OTA...")
        with context.gus() as gus:
            result = gus.run_sync(context.timeout,
                                  snapstraction.nvparam.reset_nvparams,
                                  context.target_address)
            update_cache(context)
            context.output(result)
    elif args.value is None:
        log.info("Reading NVParam %s", args.nvparam)
        with context.gus() as gus:
            result = gus.run_sync(context.timeout,
                                  snapstraction.nvparam.get_nvparam,
                                  context.target_address, args.nvparam)
            update_cache(context)
            context.output(result)
    else:
        args.value = parse_value_for_snap(args.value)
        log.info("Setting NVparam %s to %r", args.nvparam, args.value)
        with context.gus() as gus:
            result = gus.run_sync(
                context.timeout, snapstraction.nvparam.set_nvparam,
                context.target_address, args.nvparam, args.value)
            update_cache(context)
            context.output(result)


def stats(parser, context, args):
    """Get node stats and info"""
    handler = ArgumentHandler()
    handler.add_argument('stat_name')
    args = handler.parse_args(args)

    log.info("Getting stats from node %s", context.target_address)

    with context.gus() as g:
        result = g.run_sync(context.timeout, snapstraction.node.get_stats,
                            context.target_address, args.stat_name)
        update_cache(context)
        context.output(result)


def reboot(parser, context, args):
    """Reboot the target node"""
    handler = ArgumentHandler()

    handler.add_argument('delay',
                         default=None,
                         type=int,
                         nargs='?',
                         help="Optional ms to delay before rebooting.")
    log.debug("ARGS: %r", args)

    args = handler.parse_args(args)

    log.info("Trying to reboot node %s, args: %r", context.target_address,
             args)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.reboot,
                              context.target_address,
                              delay=args.delay)
        update_cache(context)
        context.output(result)


def script(parser, context, args):
    """Script Commands"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands(
        {'info': script_info,
         'load': script_load,
         'erase': script_erase})

    log.info("script commands")
    log.info("script context: %r", context)
    log.info("script args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def script_info(parser, context, args):
    """Get script-related info from the node"""
    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Getting script info from node %s", context.target_address)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.node.script_info,
                              context.target_address)
        update_cache(context)
        context.output(result)


def script_load(parser, context, args):
    """Load a SPY file on the node"""
    handler = ArgumentHandler()
    handler.add_argument('spy_file')
    args = handler.parse_args(args)

    log.info("Loading spy file %s onto node %s", args.spy_file,
             context.target_address)

    result = ota_script_upload(context, args.spy_file)

    update_cache(context)
    context.output(result)


def script_erase(parser, context, args):
    """Erase the script from the node"""
    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Erasing the script from node %s", context.target_address)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.node.script_erase,
                              context.target_address)
        update_cache(context)
        context.output(result)


def ota_script_upload(context, spy_file):
    """Load script OTA"""
    # Need to allow a lot more time for firmware uploads...
    if context.timeout < 60:
        log.warn(
            "Bumping timeout to 180 seconds, script uploads might take a while...")
        context.timeout = 180

    spy_path = os.path.join(
        os.path.curdir, os.path.expanduser(spy_file)) if spy_file else None

    widgets = [progressbar.FormatLabel('OTA Script Upload (%s ==> %s):' % (os.path.basename(spy_path), context.target_address)), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA(), ' ',
               progressbar.AdaptiveTransferSpeed()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=100)
    progress.start()

    def on_progress(percent_complete):
        progress.update(min(percent_complete, 100))

    def on_finished():
        progress.finish()

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.ota_script_upload,
                              context.target_address,
                              spy_path,
                              progress_cb=on_progress,
                              finished_cb=on_finished)
        return result


def firmware(parser, context, args):
    """Firmware Commands"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({'info': firmware_info, 'load': ota_firmware_load})

    log.info("firmware commands")
    log.info("firmware context: %r", context)
    log.info("firmware args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def firmware_info(parser, context, args):
    """Get firmware-related info from the node"""
    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Getting firmware info from node %s", context.target_address)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.firmware_info,
                              context.target_address)
        update_cache(context)
        context.output(result)


def ota_firmware_load(parser, context, args):
    """OTA firmware upgrade"""
    handler = ArgumentHandler()
    handler.add_argument('core')
    handler.add_argument('-f',
                         '--file',
                         default=None,
                         help="Override firmware file path")
    args = handler.parse_args(args)

    # Need to allow a lot more time for firmware uploads...
    if context.timeout < 60:
        log.warn(
            "Bumping timeout to 180 seconds, firmware uploads can take a while...")
        context.timeout = 180

    log.warn(
        "The target node may be unresponsive for up to 2 minutes *after* the upload completes.")
    log.warn("Do not remove power from the node during this time.")

    firmware_name = args.core if not args.file else os.path.basename(args.file)

    widgets = [progressbar.FormatLabel('OTA Firmware Load (%s ==> %s):' %
                                       (firmware_name, context.target_address)), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA(), ' ',
               progressbar.AdaptiveTransferSpeed()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=100)
    progress.start()

    def on_progress(percent_complete):
        progress.update(min(percent_complete, 100))

    def on_finished():
        progress.finish()

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.ota_firmware_upload,
                              context.target_address,
                              args.core,
                              args.file,
                              progress_cb=on_progress,
                              finished_cb=on_finished)
        update_cache(context)
        context.output(result)


def vmstat(parser, context, args):
    """Get an NVParam"""
    handler = ArgumentHandler()
    handler.add_argument('vmstat_code', type=int)
    handler.add_argument('vmstat_arg', nargs='?', default=None, type=int)
    args = handler.parse_args(args)

    log.info("Trying to get vmstat %d from node %s", args.vmstat_code,
             context.target_address)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.node.get_vmstat,
                              context.target_address, args.vmstat_code,
                              args.vmstat_arg)
        update_cache(context)
        context.output(result)


def energy(parser, context, args):
    """Do an RF energy scan"""
    handler = ArgumentHandler()

    handler.add_argument("-n",
                         "--num_queries",
                         default=1,
                         type=int,
                         help="Number of energy queries to average")

    handler.add_argument("-d",
                         "--delay",
                         default=0,
                         type=float,
                         help="delay in seconds between queries")

    args = handler.parse_args(args)

    log.info("Trying to scanEnergy from node %s", context.target_address)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.scan_energy,
                              context.target_address,
                              num=args.num_queries,
                              delay=args.delay)

    if result.keys():

        try:
            # Use stderr to spit out a little graph
            # 0, 16, 32, 50, 66, 83, 100
            ticks = [u'▁', u'▂', u'▃', u'▄', u'▅', u'▆', u'▇']

            # scale energies to 0-100
            levels = [0, 21, 42, 64, 85, 106, 128]

            sys.stderr.write('\n')

            for i in result.iterkeys():
                channel_energy = result[i]
                j = 0
                while levels[j] < channel_energy:
                    j += 1
                sys.stderr.write(ticks[j] + ticks[j] + ' ')

            sys.stderr.write('\n')

            for i in result.iterkeys():
                sys.stderr.write('%s ' % i[-2:])

            sys.stderr.write('\n')
        except:
            log.warning(
                "Can't display Unicode-graph, couldn't output unicode on this tty.")

    update_cache(context)
    context.output(result)


def intercept(parser, context, args):
    """Node output intercept commands"""
    handler = ArgumentHandler()

    # Add subcommands here. Again, the @subcmd decorator doesn't
    # quite do the right thing for us, so we have to manually register.
    handler.set_subcommands({'start': intercept_start, 'stop': intercept_stop})

    log.info("callback args: %r", args)

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def intercept_start(parser, context, args):
    """Tell the node to start sending us its output"""
    handler = ArgumentHandler()

    sources_group = handler.add_mutually_exclusive_group()
    sources_group.add_argument('--out-only', action='store_true')
    sources_group.add_argument('--err-only', action='store_true')

    args = handler.parse_args(args)

    intercept_stdout = not args.err_only
    intercept_stderr = not args.out_only

    sources = [('stdout', intercept_stdout), ('stderr', intercept_stderr)]
    log.info("Trying to start intercepting %s from node %s",
             ' and '.join([name for (name, enabled) in sources if enabled]),
             context.target_address)

    with context.gus() as gus:
        result = gus.run_sync(
            context.timeout, snapstraction.node.start_output_interceptor,
            context.target_address, intercept_stdout, intercept_stderr)
        update_cache(context)
        context.output(result)


def intercept_stop(parser, context, args):
    """Tell the node to stop sending us output"""
    handler = ArgumentHandler()

    args = handler.parse_args(args)

    sources = ['stdout', 'stderr']
    log.info("Trying to stop intercepting %s from node %s",
             ' and '.join(sources), context.target_address)
    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.stop_output_interceptor,
                              context.target_address)
        update_cache(context)
        context.output(result)


def module_type(parser, context, args):
    """Query the node's firmware version and module type"""
    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Trying to find out what module type node %s is...",
             context.target_address)
    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.node.get_node_core_module,
                              context.target_address)
        update_cache(context)
        context.output(result)


def ping(parser, context, args):
    """Unicast ping to a single node"""
    handler = ArgumentHandler()
    args = handler.parse_args(args)

    log.info("Pinging node %s...", context.target_address)
    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.node.ping,
                              context.target_address)
        update_cache(context)
        context.output(result)


def move(parser, context, args):
    """
Move the target node to the target network.

Assumes you can already talk to the target.
If the target is the bridge, then update the profile's "last used" network.
    """

    handler = ArgumentHandler()
    handler.add_argument('target_network', help="Network to move the node to")
    args = handler.parse_args(args)

    log.info("Moving node %s to %s...", context.target_address,
             args.target_network)

    try:
        target_network = Network.load(args.target_network)
    except:
        log.error("Unable to load target network %s", args.target_network)
        exit(1)

    with context.gus() as gus:
        context.target_address = clean_up(context.target_address,
                                          gus.bridge_address)
        result = gus.run_sync(context.timeout,
                              snapstraction.node_finder.move_target_to_network,
                              context.target_address, target_network)

        if context.target_address == gus.bridge_address:
            # Update the profile's network to where we just moved the bridge node
            context.profile.network = args.target_network
            context.profile.save()

        update_cache(context)
        context.output(result)
